<?php
// $Id: trevi.admin.inc,v 1.1.2.25 2011/02/07 21:49:06 xano Exp $

/**
 * @file
 *   Administration pages.
 */

/**
 * List text sources.
 */
function trevi_page_source_list() {
  drupal_add_css(drupal_get_path('module', 'trevi') . '/css/trevi.css');

  $rows = array();
  $sources = array();
  foreach (trevi_source_load_all() as $source) {
    $sources[$source->group][] = $source;
  }
  ksort($sources);
  foreach ($sources as $group => $group_sources) {
    $rows[] = array(
      array(
        'data' => $group,
        'colspan' => 3,
        'header' => TRUE,
      ),
    );
    foreach ($group_sources as $source) {
      $description = $source->description ? '<div class="description">' . $source->description . '</div>' : NULL;
      $rows[] = array(
        array(
          'data' => $source->title . $description,
          'class' => 'trevi-source-list-title',
        ),
        $source->methods ? t('enabled') : NULL,
        l(t('configure'), 'admin/settings/trevi/configure/' . $source->name),
      );
    }
  }
  if (!$rows) {
    $rows[] = array(
      array(
        'data' => t('There are no text sources yet. Enable or configure modules that provide them.'),
        'colspan' => 3,
      ),
    );
  }
  $header = array(
    t('Text source'),
    t('Enabled'),
    t('Operations'),
  );

  return theme('table', $header, $rows, array(
    'id' => 'trevi-source-list',
  ));
}

/**
 * Create all elements for text source configuration.
 */
function trevi_form_source_configuration(array $form_state, treviSource $source) {
  drupal_set_title(t('Configure %title for @group', array(
    '%title' => $source->title,
    '@group' => $source->group,
  )));

  $methods = trevi_method_load_all();

  $form['description'] = array(
    '#value' => $source->description,
  );

  $languages = trevi_languages();
  $form['language'] = array(
    '#type' => 'item',
    '#title' => format_plural(count($source->languages), t('Language'), t('Languages')),
    '#value' => implode(', ', array_intersect_key($languages, array_flip($source->languages))),
  );

  // Blacklist and whitelist configuration.
  $form['lists'] = array(
    '#type' => 'trevi_source_list_configuration',
    '#trevi_source_name' => $source->name,
  );

  // Metadata reviews.
  if ($source->method_types & TREVI_METADATA) {
    $metadata_options = array();
    foreach ($methods as $method) {
      if ($method->type == TREVI_METADATA && (!$method->reference || $method->reference == $source->reference)) {
        $description = $method->description ? '<div class="description">' . $method->description . '</div>' : NULL;
        $metadata_options[$method->name] = $method->title . $description;
      }
    }
    if (empty($metadata_options)) {
      $element['metadata'] = array(
        '#type' => 'item',
        '#title' => t('Metadata'),
        '#value' => t('There are no metadata reviews yet. Enable or configure modules that provide them.'),
      );
    }
    else {
      $form['metadata'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Metadata'),
        '#options' => $metadata_options,
        '#default_value' => array_keys($source->methods),
      );
    }
  }

  // Defusers. We assume there is at least one.
  $defuser_options = array();
  foreach (trevi_defuser_load_all() as $defuser_name => $defuser) {
    $defuser_options[$defuser_name] = $defuser->title;
  }
  $form['defusers'] = array(
    '#title' => t('Ignore'),
    '#type' => 'checkboxes',
    '#default_value' => array_keys($source->defusers),
    '#options' => $defuser_options,
  );

  // Buttons.
  $form['buttons'] = array(
    'submit' => array(
      '#type' => 'submit',
      '#value' => t('Save configuration'),
    ),
    'cancel' => array(
      '#value' => l(t('Cancel'), 'admin/settings/trevi', array(
        'attributes' => array(
          'class' => 'trevi-source-configuration-link',
        ),
      )),
    ),
  );

  return $form;
}

/**
 * Form submit callback for trevi_form_source_configuration().
 *
 * One could argue this function should not execute SQL queries directly. The
 * reason for this is that text source configuration through forms has some
 * extras on top of the regular API. The form requires all review methods to be
 * shown and processed, while the API only allows enabled methods to exist
 * within a text source.
 */
function trevi_form_source_configuration_submit(array $form, array &$form_state) {
  $values = $form_state['values'];
  $source = trevi_source_load($form['lists']['#trevi_source_name']);

  db_query("DELETE FROM {trevi_defuser} WHERE source_name = '%s'", $source->name);
  db_query("DELETE FROM {trevi_method} WHERE source_name = '%s'", $source->name);

  // Defusers.
  foreach ($values['defusers'] as $defuser_name => $value) {
    if ($value) {
      db_query("INSERT INTO {trevi_defuser} (source_name, name) VALUES ('%s', '%s')", $source->name, $defuser_name);
    }
  }

  // Review methods.
  $weights = array();
  foreach (trevi_method_load_all() as $method) {
    if (TREVI_LIST & $method->type && isset($values['weight_' . $method->name])) {
      $weights[] = $values['weight_' . $method->name];
      db_query("INSERT INTO {trevi_method} (source_name, name, weight, enabled) VALUES ('%s', '%s', %d, %d)", $source->name, $method->name, end($weights), $values['enabled_' . $method->name]);
    }
  }
  if ($source->method_types & TREVI_METADATA) {
    $weight = max($weights);
    foreach ($values['metadata'] as $method_name => $value) {
      if ($value) {
        $weight++;
        db_query("INSERT INTO {trevi_method} (source_name, name, weight, enabled) VALUES ('%s', '%s', %d, 1)", $source->name, $method_name, $weight);
      }
    }
  }

  $form_state['redirect'] = 'admin/settings/trevi';
  drupal_set_message(t('The configuration for %title for @group has been saved.', array(
    '%title' => $source->title,
    '@group' => $source->group,
  )));
}

/**
 * Form process callback for a trevi_source_list_configuration element.
 */
function trevi_process_source_list_configuration($element) {
  $source = trevi_source_load($element['#trevi_source_name']);
  $methods = trevi_method_load_all();

  $weights = array_fill_keys(array_keys($methods), 0);
  $result = db_query("SELECT name, weight FROM {trevi_method} WHERE source_name = '%s' ORDER BY weight, name ASC", $source->name);
  while ($method_data = db_fetch_object($result)) {
    $weights[$method_data->name] = $method_data->weight;
  }
  // If we don't sort by weight, existing weights will not be respected when
  //  dragging rows.
  asort($weights);
  $delta = round(count($methods) / 2);
  foreach (array_keys($weights) as $method_name) {
    if (isset($methods[$method_name])) {
      $method = $methods[$method_name];
      if (TREVI_LIST & $method->type
      && $source->method_types & $method->type
      && (!$method->reference || $method->reference == $source->reference)) {
        $class = 'trevi-method-disabled';
        $weight = 0;
        if (isset($weights[$method->name])) {
          $class = 'trevi-method-enabled';
          $weight = $weights[$method->name];
        }
        $element[$method->name]['weight_' . $method->name] = array(
          '#type' => 'weight', 
          '#default_value' => $weights[$method->name],
          '#delta' => $delta,
          '#attributes' => array(
            'class' => 'trevi-method-weight ' . $class,
          ),
        );
        $element[$method->name]['enabled_' . $method->name] = array(
          '#type' => 'checkbox',
          '#title' => t('Enabled'),
          '#default_value' => isset($source->methods[$method->name]), 
        );
      }
    }
  }

  return $element;
}

function theme_trevi_source_list_configuration($element) {
  $methods = trevi_method_load_all();

  // Theme blacklists and whitelists into table rows.
  $languages = trevi_languages();
  $types = array(
    TREVI_BLACKLIST => t('Blacklist'),
    TREVI_WHITELIST => t('Whitelist'),
  );
  $rows = array();
  foreach (element_children($element) as $method_name) {
    $method = $methods[$method_name];
    $description = $method->description ? '<div class="description">' . $method->description . '</div>' : '';
    // Labels? Where we're going, we don't need labels.
    unset($element[$method_name]['enabled_' . $method->name]['#title']);
    // Unset #printed to make sure the elements are themed. This is a fix for a
    // bug in drupal_render().
    unset($element[$method_name]['enabled_' . $method->name]['#printed']);
    unset($element[$method_name]['weight_' . $method->name]['#printed']);
    $rows[] = array(
      'data' => array(
        $method->title . $description,
        drupal_render($element[$method_name]['enabled_' . $method->name]),
        array(
          'data' => drupal_render($element[$method_name]['weight_' . $method->name]),
          'class' => 'js-hide',
        ),
        $types[$method->type],
        implode(', ', array_intersect_key($languages, array_flip($method->languages))),
      ),
      'class' => 'draggable',
    );
  }
  // Display an error message if there are no list methods.
  if (empty($rows)) {
    $rows = array(
      array(
        array(
          'data' => t('There are no blacklists available for this text source. Enable or configure modules that provide them.'),
          'colspan' => 5,
        ),
      ),
    );
  }

  $header = array(
    t('Review method'),
    t('Enabled'),
    array(
      'data' => t('Weight'),
      'class' => 'js-hide',
    ),
    t('Type'),
    t('Languages'),
  );

  drupal_add_tabledrag('trevi-source-configuration', 'order', 'sibling', 'trevi-method-weight');

  return theme('table', $header, $rows, array(
    'id' => 'trevi-source-configuration',
  ));
}