<?php
// $Id: trevi.result.inc,v 1.1.2.12 2011/01/20 00:24:38 xano Exp $

/**
 * @file
 *   Render trevi() results in a string for display.
 */

/**
 * Render a review result into a string.
 *
 * @param $string string
 *   The string to render the result into.
 * @param $result treviResult
 *   The output from trevi().
 *
 * @return string
 *   The rendered string.
 */
function trevi_view_highlights($string, treviResult $result) {
  drupal_add_css(drupal_get_path('module', 'trevi') . '/css/trevi.css');

  $characters = array();
  foreach (str_split($string) as $character) {
    $characters[] = new treviCharacter($character);
  }

  // Process existing matches.
  foreach ($result->existing as $class => $existing) {
    foreach ($existing as $position => $comments) {
      $characters[$position]->classes[] = $class;
      if ($comments) {
        $characters[$position]->comments += $comments;
      }
    }
  }

  // Process nonexisting matches. Put them in a buffer array first, because
  // they are first processed by class and then by position.
  $insertions = array();
  foreach ($result->nonexisting as $class => $nonexisting) {
    // Loop through the removals in reverse because they modify character
    // positions.
    krsort($nonexisting);
    foreach ($nonexisting as $position => $fragments) {
      if (!isset($insertions[$position])) {
        $insertions[$position] = array();
      }
      foreach ($fragments as $text) {
        $insertions[$position][] = new treviCharacter($text, array($class));
      }
    }
  }
  krsort($insertions);
  foreach ($insertions as $position => $insertion) {
    $prefix = array_slice($characters, 0, $position);
    $suffix = array_slice($characters, $position);
    $characters = array_merge($prefix, $insertion, $suffix);
  }

  // Render the highlights.
  $rendered = '';
  foreach ($characters as $character) {
    // Comments are optional, so we check for the match class.
    if ($character->classes) {
      $character->classes = array_unique($character->classes);
      $rendered .= theme('trevi_highlight', $character);
    }
    else {
      $rendered .= $character->text;
    }
  }

  return $rendered;
}

class treviCharacter {
  public $text = '';
  public $comments = array();
  public $classes = array();

  function __construct($text, array $classes = array()) {
    $this->text = $text;
    $this->classes = $classes;
  }
}

/**
 * Render review metadata.
 *
 * @param $result treviResult
 *   The result object from which to render the medata.
 *
 * @return string
 */
function trevi_view_metadata(treviResult $result) {
  $rows = array();
  foreach ($result->metadata as $metadata) {
    $rows[] = array($metadata['#title'], $metadata['#value']);
  }
  return theme('table', array(), $rows);
}

/**
 * Highlight a character.
 *
 * @param $character array
 *   Information about the character to highlight.
 *
 * @return string
 *   The highlighted character.
 */
function theme_trevi_highlight($character) {
  $attributes = array(
    'class' => 'trevi-highlight',
  );
  foreach ($character->classes as $class) {
    $attributes['class'] .= " trevi-$class";
  }
  if ($character->comments) {
    $attributes['class'] .= " trevi-comment";
    $attributes['title'] = strip_tags(implode('. ', $character->comments));
  }
  return '<span' . drupal_attributes($attributes) . '>' . $character->text . '</span>';
}