<?php
// $Id: trevilist.admin.inc,v 1.1.2.11 2011/01/23 13:53:13 xano Exp $

/**
 * @file
 *   Administration pages.
 */

/**
 * List all lists.
 */
function trevilist_list() {
  $rows = array(
    TREVI_BLACKLIST => array(),
    TREVI_WHITELIST => array(),
  );
  $messages = array(
    TREVI_BLACKLIST => t('There are no blacklists yet. <a href="!add_form">Add a new blacklist</a>.', array('!add_form' => url('admin/settings/trevi/trevilist/add-blacklist'))),
    TREVI_WHITELIST => t('There are no whitelists yet. <a href="!add_form">Add a new whitelist</a>.', array('!add_form' => url('admin/settings/trevi/trevilist/add-whitelist'))),
  );
  $languages = trevi_languages();
  foreach (trevilist_load() as $method) {
    $language = implode(array_intersect_key($languages, array_flip($method->languages)));
    $rows[$method->type][] = array(
      $method->title,
      $language,
      l(t('edit'), 'admin/settings/trevi/trevilist/edit/' . $method->tlmid),
      l(t('delete'), 'admin/settings/trevi/trevilist/delete/' . $method->tlmid),
      l(t('export'), 'admin/settings/trevi/trevilist/export/' . $method->tlmid),
    );
  }
  foreach (array_keys($rows) as $type) {
    if (!$rows[$type]) {
      $rows[$type][] = array(
        array(
          'data' => $messages[$type],
          'colspan' => 5,
        ),
      );
    }
  }

  $header[TREVI_BLACKLIST] = array(
    t('Blacklists'),
    t('Languages'),
    array(
      'data' => t('Operations'),
      'colspan' => 3,
    ),
  );
  $header[TREVI_WHITELIST] = $header[TREVI_BLACKLIST];
  $header[TREVI_WHITELIST][0] = t('Whitelists');

  return theme('table', $header[TREVI_BLACKLIST], $rows[TREVI_BLACKLIST]) . theme('table', $header[TREVI_WHITELIST], $rows[TREVI_WHITELIST]);
}

function trevilist_page_add($type) {
  return drupal_get_form('trevilist_form', new trevilistMethod(array(
    'type' => $type,
  )));
}

/**
 * Add or edit a list.
 */
function trevilist_form(array &$form_state, trevilistMethod $method) {
  $form['#redirect'] = 'admin/settings/trevi/trevilist';
  $form['#trevilist_method'] = $method;
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $method->title,
    '#required' => TRUE,
  );
  $languages = trevi_languages();
  unset($languages['und'], $languages['zxx']);
  $form['languages'] = array(
    '#type' => 'select',
    '#title' => t('Languages'),
    '#description' => t('Leave empty if this list is language independent.'),
    '#default_value' => $method->languages,
    '#options' => $languages,
    '#multiple' => TRUE,
    '#size' => 5,
  );
  $form['keywords'] = array(
    '#type' => 'trevilist_keywords',
    '#trevilist_method' => $method,
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['buttons']['update'] = array(
    '#type' => 'submit',
    '#value' => t('Add/remove keyword fields'),
    '#name' => 'update'
  );

  return $form;
}

/**
 * Form validate handler for trevilist_form().
 */
function trevilist_form_validate(array $form, array &$form_state) {
  $keywords = $form_state['values']['keywords'];

  // Make sure there's at least one keyword.
  $keywords_exist = FALSE;
  $i = 0;
  while (isset($keywords['keyword_' . $i])) {
    if (strlen($keywords['keyword_' . $i])) {
      $keywords_exist = TRUE;
    }
    $i++;
  }
  if (!$keywords_exist) {
    form_set_error('', t('At least one keyword is required.'));
  }

  // Validate regular expressions.
  $i = 0;
  while (isset($keywords['keyword_' . $i])) {
    if (strlen($keywords['keyword_' . $i]) && $keywords['match_' . $i] == TREVILIST_PCRE) {
      if (@preg_match($keywords['keyword_' . $i], '') === FALSE) {
        form_set_error('keywords][keyword_' . $i, t('%pattern is not a valid <a href="@url">Perl-compatible regular expression</a>.', array(
          '%pattern' => $keywords['keyword_' . $i],
          '@url' => 'http://php.net/manual/en/reference.pcre.pattern.syntax.php',
        )));
      }
    }
    $i++;
  }
}

/**
 * Form submit handler for trevilist_form().
 */
function trevilist_form_submit(array $form, array &$form_state) {
  $values = $form_state['values'];
  $method = $form['#trevilist_method'];
  $method->keywords = array();
  $method->title = $values['title'];
  $method->languages = empty($values['languages']) ? array('zxx') : $values['languages'];

  $keywords = $values['keywords'];
  $i = 0;
  while (isset($keywords['keyword_' . $i])) {
    if (strlen($keywords['keyword_' . $i])) {
      $comment = isset($keywords['comment_' . $i]) ? $keywords['comment_' . $i] : NULL;
      $method->keywords[] = new trevilistKeyword($keywords['keyword_' . $i], $comment, $keywords['match_' . $i]);
    }
    $i++;
  }

  if ($form_state['clicked_button']['#name'] == 'update') {
    $form_state['rebuild'] = TRUE;
  }
  else {
    trevilist_save($method);
    drupal_set_message(t('%title has been saved.', array('%title' => $method->title)));
  }
}

/**
 * Delete a trevilistMethod.
 */
function trevilist_form_delete(array &$form_state, trevilistMethod $method) {
  $form['#redirect'] = 'admin/settings/trevi/trevilist';
  $form['#trevilist_method'] = $method;
  $question = $method->type == TREVI_BLACKLIST ? t('Are you sure you want to delete blacklist %title?', array('%title' => $method->title)) : t('Are you sure you want to delete whitelist %title?', array('%title' => $method->title));

  return confirm_form($form, $question, 'admin/settings/trevilist/', NULL, t('Delete'));
}

/**
 * Form submit handler for trevilist_form_delete().
 */
function trevilist_form_delete_submit(array $form, array &$form_state) {
  $method = $form['#trevilist_method'];
  trevilist_delete($method->tlmid);
  drupal_set_message(t('%title has been deleted.', array('%title' => $method->title)));
}

/**
 * Import a trevilistMethod.
 */
function trevilist_form_import(array &$form_state) {
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Override title'),
    '#description' => t('Optionally use a different title.'),
  );
  $form['code'] = array(
    '#type' => 'textarea',
    '#title' => t('Paste list method code'),
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );

  return $form;
}

/**
 * Form validate handler for trevilist_form_import().
 */
function trevilist_form_import_validate(array $form, array &$form_state) {
  $values = &$form_state['values'];
  if (strlen($values['code']) && !$values['method'] = trevilist_import($values['code'])) {
    form_set_error('code', t('The code is no valid review method.'));
  }
  elseif (strlen($values['title'])) {
    $values['method']->title = $values['title'];
  }
}

/**
 * Form submit handler for trevilist_form_import().
 */
function trevilist_form_import_submit(array $form, array &$form_state) {
  $method = $form_state['values']['method'];
  if (is_object($method)) {
    trevilist_save($method);
    drupal_set_message(t('%title has been imported.', array('%title' => $method->title)));
  }
  $form_state['redirect'] = 'admin/settings/trevi/trevilist';
}

/**
 * Form process handler for trevilist_keywords elements.
 */
function trevilist_form_process_trevilist_keywords(array $element) {
  $method = $element['#trevilist_method'];
  $i = -1;
  if (count($method->keywords())) {
    foreach ($method->keywords() as $keyword) {
      $i++;
      trevilist_keyword_elements($element, $i, $method, $keyword);
    }
  }
  foreach (range($i + 1, $i + 10) as $j) {
      trevilist_keyword_elements($element, $j, $method);
  }

  $element['#description'] = $method->type == TREVI_BLACKLIST ? t('Any words or phrases that should not exist in texts, with additional comments, such as possible improvements.') : t('Any words or phrases that are always allowed in texts.');
  $element['#tree'] = TRUE;

  return $element;
}

function trevilist_keyword_elements(array &$element, $i, trevilistMethod $method, trevilistKeyword $keyword = NULL) {
  $element['keyword_' . $i] = array(
    '#type' => 'textfield',
    '#default_value' => $keyword ? $keyword->keyword : NULL,
    '#size' => 30,
  );
  $element['match_' . $i] = array(
    '#type' => 'select',
    '#default_value' => $keyword ? $keyword->type : TREVILIST_CASE_INSENSITIVE,
    '#options' => array(
      TREVILIST_CASE_INSENSITIVE => t('Case insensitive'),
      TREVILIST_CASE_SENSITIVE => t('Case sensitive'),
      TREVILIST_PCRE => t('PCRE'),
    ),
  );
  if ($method->type == TREVI_BLACKLIST) {
    $element['comment_' . $i] = array(
      '#type' => 'textarea',
      '#default_value' => $keyword ? $keyword->comment : NULL,
      '#rows' => 1,
    );
  }
}

/**
 * Form theme handler; theme a trevilist_keywords element.
 */
function theme_trevilist_keywords(array $element) {
  $i = 0;
  $rows = array();
  while (isset($element['keyword_' . $i])) {
    unset($element['keyword_' . $i]['#printed'], $element['match_' . $i]['#printed']);
    $row = array(
      drupal_render($element['keyword_' . $i]),
      drupal_render($element['match_' . $i]),
    );
    if (isset($element['comment_' . $i])) {
    unset($element['comment_' . $i]['#printed']);
      $row[] = drupal_render($element['comment_' . $i]);
    }
    $rows[] = $row;

    $i++;
  }
  $header = array(t('Keyword'), t('Match type'));
  if ($element['#trevilist_method']->type == TREVI_BLACKLIST) {
    $header[] = t('Comment');
  }

  return theme('form_element', $element, theme('table', $header, $rows));
}

/**
 * Export a trevilistMethod as a file download.
 *
 * @param $method trevilistMethod
 *   The method to export.
 */
function trevilist_export(trevilistMethod $method) {
  drupal_set_header('Content-Type: text/plain; charset=UTF-8');
  drupal_set_header('Content-Disposition: attachment; filename=' . trevilist_export_filename($method->title));

  $title = str_replace("'", "\'", $method->title);
  $type = $method->type == TREVI_BLACKLIST ? 'TREVI_BLACKLIST' : 'TREVI_WHITELIST';
  $languages = array();
  foreach ($method->languages as $language) {
    $languages[] = "'$language'";
  }
  $languages = implode(', ', $languages);
  $keywords = array();
  foreach ($method->keywords() as $keyword) {
    $buffer = "\n  new trevilistKeyword('" . str_replace("'", "\'", $keyword->keyword) . "'";
    if ($keyword->comment) {
      $buffer .= ", '" . str_replace("'", "\'", $keyword->comment) . "'";
    }
    if ($keyword->type != TREVILIST_CASE_INSENSITIVE) {
      $keyword_type = $keyword->type == TREVILIST_PCRE ? 'TREVILIST_PCRE' : 'TREVILIST_CASE_SENSITIVE';
      $buffer .= ", '$keyword_type'";
    }
    $keywords[] = $buffer . ')';
  }
  $keywords = implode(', ', $keywords);
  $export = <<<EOT
\$method = new trevilistMethod(array(
  'title' => '$title',
  'type' => $type,
  'languages' => array($languages),
  'keywords' => array($keywords
  ),
));
EOT;

  echo $export;
}

/**
 * Return an array with characters that cannot appear in filenames.
 *
 * @return array
 */
function trevilist_export_filename_illegal_characters() {
  return array(' ', '/', '\\', '?', '%', '*', ':', '|', '\"', '<', '>', '.');
}

/**
 * Create the filename for an exported method.
 *
 * @param $source string
 *   The string to use for the filename. The method's title for instance.
 */
function trevilist_export_filename($source) {
  return str_replace(trevilist_export_filename_illegal_characters(), '-', strtolower(trim($source))) . '.txt';
}